# -*- coding: utf-8 -*-

import os
from .helper import HOME
from .env_helper import append_line_if_not_exists, load_var_value_from_shell_script


class EnvSecret(object):
    """
    Allow to load secret information from environment variable.
    """

    pysecret_file = ".bashrc_pysecret"
    pysecret_script = os.path.join(HOME, pysecret_file)

    bashrc_script = os.path.join(HOME, ".bashrc")
    bash_profile_script = os.path.join(HOME, ".bash_profile")
    zshrc_script = os.path.join(HOME, ".zshrc")
    config_fish_script = os.path.join(HOME, ".config", "fish", "config.fish")

    def export_cmd_text(self, var, value):
        """
        create ``export VAR="VALUE"`` command text
        """
        return 'export {var}="{value}"'.format(var=var, value=value)

    def set(self, var, value, temp=False):
        """
        Set value.

        :type var: str
        :param var:

        :type value: str
        :param value:

        :type temp: bool
        :param temp: if True, then will not write ``export var="value"`` to pysecret file.

        :return: None
        """
        os.environ[var] = str(value)
        if temp is False:
            append_line_if_not_exists(
                self.pysecret_script, self.export_cmd_text(var, value))

    @property
    def environ(self):
        return os.environ

    def get(self, var):
        """
        Get value.

        :type var: str
        :param var:

        :rtype: str
        :return:
        """
        return self.environ[var]

    def unset(self, var):  # pragma: no cover
        raise NotImplementedError("not implemented yet!")

    # -- add ``source ~/<pysecret_file>`` to bash profile file
    @property
    def source_pysecret_command(self):
        return "source ~/{}".format(self.pysecret_file)

    def apply_source_pysecret_to_bashrc(self):  # pragma: no cover
        append_line_if_not_exists(
            self.bashrc_script, self.source_pysecret_command
        )

    def apply_source_pysecret_to_bash_profile(self):  # pragma: no cover
        append_line_if_not_exists(
            self.bash_profile_script, self.source_pysecret_command
        )

    def apply_source_pysecret_to_zshrc(self):  # pragma: no cover
        append_line_if_not_exists(
            self.zshrc_script, self.source_pysecret_command
        )

    def apply_source_pysecret_to_config_fish(self):  # pragma: no cover
        append_line_if_not_exists(
            self.config_fish_script, self.source_pysecret_command
        )

    def load_pysecret_script(self):
        environ = load_var_value_from_shell_script(self.pysecret_script)
        for key, value in environ.items():
            os.environ[key] = value



# -*- coding: utf-8 -*-

"""
pysecret is a tiny library to allow developer load secret information safely.

- from environment variable
- from json file
- from AWS Secret Manager and Key Management Service
"""

from ._version import __version__

__short_description__ = "utility tool that load secret information safely."
__license__ = "MIT"
__author__ = "Sanhe Hu"
__author_email__ = "husanhe@gmail.com"
__github_username__ = "MacHu-GWU"

try:
    from .helper import home_file_path
    from .env import EnvSecret
    from .js import JsonSecret, DEFAULT_JSON_SECRET_FILE
except ImportError:  # pragma: no cover
    pass
except Exception as e:  # pragma: no cover
    raise e

try:
    from .aws import AWSSecret
except ImportError:  # pragma: no cover
    pass
except Exception as e:  # pragma: no cover
    raise e



# -*- coding: utf-8 -*-

"""
This module provides high level API for AWS Key Management Service and
AWS Secret Manager.
"""

import base64
import json

import boto3

from .js_helper import get_value


class AWSSecret(object):
    """
    An AWS Secret syntax simplifier class.
    """

    def __init__(self,
                 aws_access_key_id=None,
                 aws_secret_access_key=None,
                 aws_session_token=None,
                 region_name=None,
                 botocore_session=None,
                 profile_name=None):
        self.ses = boto3.Session(
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            aws_session_token=aws_session_token,
            region_name=region_name,
            botocore_session=botocore_session,
            profile_name=profile_name,
        )
        self._kms_client = None
        self._sm_client = None
        self._ssm_client = None
        self.secret_cache = dict()
        self.parameter_cache = dict()

    @property
    def kms_client(self):
        if self._kms_client is None:
            self._kms_client = self.ses.client("kms")
        return self._kms_client

    @property
    def sm_client(self):
        if self._sm_client is None:
            self._sm_client = self.ses.client("secretsmanager")
        return self._sm_client

    @property
    def ssm_client(self):
        if self._ssm_client is None:
            self._ssm_client = self.ses.client("ssm")
        return self._ssm_client

    def kms_encrypt(self, kms_key_id, text):
        """
        Use KMS key to encrypt a short text.

        :type kms_key_id: str
        :param kms_key_id:

        :type text: str
        :param text:

        :rtype: str
        """
        return base64.b64encode(
            self.kms_client.encrypt(
                KeyId=kms_key_id,
                Plaintext=base64.b64encode(text.encode("utf-8")),
            )["CiphertextBlob"]
        ).decode("utf-8")

    def kms_decrypt(self, text):
        """
        Use KMS key to decrypt a short text.

        :type text: str
        :param text: text to decrypt

        :rtype: str
        """
        return base64.b64decode(self.kms_client.decrypt(
            CiphertextBlob=base64.b64decode(text.encode("utf-8"))
        )["Plaintext"]).decode("utf-8")

    def get_secret_value(self, secret_id, key):
        """
        Fetch a specific secret value

        :type secret_id: str
        :param secret_id: aws secret id

        :type key: str
        :param key: secret value dictionary key

        :rtype: str
        :return: secret value in string
        """
        if self.secret_cache.get(secret_id) is None:
            response = self.sm_client.get_secret_value(SecretId=secret_id)
            if "SecretString" in response:
                secret = response["SecretString"]
            else:
                decoded_binary_secret = base64.b64decode(response["SecretBinary"])
                secret = decoded_binary_secret
            data = json.loads(secret)
            self.secret_cache[secret_id] = data
        else:
            data = self.secret_cache[secret_id]
        return get_value(data, key)

    def deploy_secret(self,
                      name,
                      secret_data,
                      description=None,
                      kms_key_id=None,
                      tags=None):
        """
        Create or Update a AWS Secret.

        - create_secret: https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/secretsmanager.html#SecretsManager.Client.create_secret
        - update_secret: https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/secretsmanager.html#SecretsManager.Client.update_secret
        """
        if tags is None:
            tags = dict()
        Tags = [
            {"Key": k, "Value": v}
            for k, v in tags.items()
        ]

        create_or_update_secret_kwargs = {"SecretString": json.dumps(secret_data)}
        if description:
            create_or_update_secret_kwargs["Description"] = description
        if kms_key_id:
            create_or_update_secret_kwargs["KmsKeyId"] = kms_key_id
        if Tags:
            create_or_update_secret_kwargs["Tags"] = Tags

        try:
            create_or_update_secret_kwargs["Name"] = name
            response = self.sm_client.create_secret(**create_or_update_secret_kwargs)
            self.secret_cache[name] = None
            return response
        except Exception as e:
            if type(e).__name__ == "ResourceExistsException":
                create_or_update_secret_kwargs.pop("Name")
                create_or_update_secret_kwargs["SecretId"] = name
                response = self.sm_client.update_secret(**create_or_update_secret_kwargs)
                self.secret_cache[name] = None
                return response
            else:
                raise e

    def get_parameter_value(self, parameter_name, key):
        """
        Fetch a specific parameter value

        :type parameter_name: str
        :param parameter_name: aws system manager parameter name

        :type key: str
        :param key: parameter value dictionary key

        :rtype: str
        :return: parameter value in string
        """
        if self.parameter_cache.get(parameter_name) is None:
            response = self.ssm_client.get_parameter(Name=parameter_name)
            if "Parameter" in response:
                string_value = response["Parameter"]["Value"]
            else:
                raise ValueError("Not a valid get_parameter response!")
            data = json.loads(string_value)
            self.parameter_cache[parameter_name] = data
        else:
            data = self.parameter_cache[parameter_name]
        return get_value(data, key)

    def deploy_parameter(self,
                         name,
                         parameter_data,
                         description=None,
                         kms_key_id=None,
                         policies=None,
                         tags=None):
        if tags is None:
            tags = dict()
        Tags = [
            {"Key": k, "Value": v}
            for k, v in tags.items()
        ]
        put_paramters_kwargs = {
            "Name": name,
            "Value": json.dumps(parameter_data),
            "Type": "String",
            "Overwrite": True,
            "Tier": "Intelligent-Tiering",
        }
        if description:
            put_paramters_kwargs["Description"] = description
        if kms_key_id:
            put_paramters_kwargs["Type"] = "SecureString"
            put_paramters_kwargs["KeyId"] = kms_key_id
        if policies:
            put_paramters_kwargs["Policies"] = policies
        if Tags:
            put_paramters_kwargs["Tags"] = Tags

        response = self.ssm_client.put_parameter(**put_paramters_kwargs)
        self.parameter_cache[name] = None
        return response


if __name__ == "__main__":
    aws_profile = "sanhe"
    kms_key_id = "a1679e4b-f415-4aa6-9637-5bee18f9eb64"

    aws = AWSSecret(profile_name=aws_profile)
    secret = "Hello World"
    encrypted_text = aws.kms_encrypt(kms_key_id, secret)
    decrypted_text = aws.kms_decrypt(encrypted_text)
    assert secret != encrypted_text
    assert secret == decrypted_text

    secret_name = "dev/learn-secret-manager"
    assert aws.get_secret_value(secret_name, "a") == "1"



# -*- coding: utf-8 -*-

import os
from superjson import json
from .helper import HOME
from .singleton import CachedSpam
from .js_helper import create_json_if_not_exists, set_value, get_value, del_key

DEFAULT_JSON_SECRET_FILE = os.path.join(HOME, ".pysecret.json")


class JsonSecret(CachedSpam):
    """

    """
    settings_uuid_field = "secret_file"

    def __real_init__(self, secret_file=DEFAULT_JSON_SECRET_FILE):
        self.secret_file = secret_file
        create_json_if_not_exists(self.secret_file)
        with open(self.secret_file, "rb") as f:
            self.data = json.loads(f.read().decode("utf-8"))

    def set(self, json_path, value):
        set_value(self.data, json_path, value)
        json.dump(self.data, self.secret_file,
                  pretty=True, ensure_ascii=False, overwrite=True, verbose=False)

    def get(self, json_path):
        return get_value(self.data, json_path)


    def unset(self, json_path):
        return del_key(self.data, json_path)



# -*- coding: utf-8 -*-

import os


def create_json_if_not_exists(path):  # pragma: no cover
    if not os.path.exists(path):
        with open(path, "wb") as f:
            f.write("{}".encode("utf-8"))


def set_value(data, json_path, value):
    paths = json_path.split(".")
    path_length = len(paths)
    for ind, key in enumerate(paths):
        if ind == (path_length - 1):
            data[key] = value
        else:
            data.setdefault(key, {})
            data = data[key]
    return data


def get_value(data, json_path):
    paths = json_path.split(".")
    value = data
    for key in paths:
        value = value[key]
    return value


def del_key(data, json_path):
    paths = json_path.split(".")
    for key in paths[:-1]:
        data = data[key]
    del data[paths[-1]]



# -*- coding: utf-8 -*-

"""Console script for pysecret."""
import sys
import click


@click.command()
def main(args=None):
    """Console script for pysecret."""
    click.echo("Replace this message by putting your code into "
               "pysecret.cli.main")
    click.echo("See click documentation at http://click.pocoo.org/")
    return 0


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



# -*- coding: utf-8 -*-


import weakref


class CachedSpamManager(object):
    """
    Instance cache manager.
    """
    cached_klass = None

    def __init__(self):
        self._cache = weakref.WeakValueDictionary()

    def new(self, name, *args, **kwargs):
        """
        Factory method.
        """
        if name not in self._cache:
            temp = self.cached_klass._new(name, *args, **kwargs)
            self._cache[name] = temp
        else:
            temp = self._cache[name]
        return temp

    def clear(self):
        self._cache.clear()


class Spam(object):
    def __init__(self, name, *args, **kwargs):
        msg = "Can't instantiate directly, use Cached{}Manager.new(name, ...) instead." \
            .format(self.__class__.__name__)
        raise RuntimeError(msg)

    # Alternate constructor
    @classmethod
    def _new(cls, name, *args, **kwargs):
        """
        User custom instance constructor.

        implement constructor this way::

            def _new(cls, name, *args, **kwargs):
                self = cls.__new__(cls)
                # put __init__(...) logic here
                self.name = name
                return self
        """
        raise NotImplementedError



# -*- coding: utf-8 -*-

import os
import re

export_pattern = re.compile('export [a-zA-Z0-9_]{1,128}="[a-zA-Z0-9_]{1,128}"')
"""Limitation, key, value length can't be greater than 128. 
"""


def append_line_if_not_exists(path, line):
    """
    Append a line to a text file at `path` if the line not exists in its content.

    :type path: str
    :param path: text file absolute path

    :type path: str
    :param line: text string.

    :return: None
    """
    if not os.path.exists(path):  # pragma: no cover
        with open(path, "wb") as f:
            f.write("".encode("utf-8"))

    if "\n" in line:  # pragma: no cover
        raise ValueError("'\\n' should not be in the line")

    # check if the line exists
    striped_line = line.strip()
    with open(path, "rb") as f:
        content = f.read().decode("utf-8")
        flag_endswith_new_line = content.endswith(
            "\n") or content.endswith("\n\r")
        for line_ in content.split("\n"):
            if striped_line in line_:
                return

    with open(path, "ab") as f:
        if flag_endswith_new_line:  # pragma: no cover
            f.write((line + "\n").encode("utf-8"))
        else:  # pragma: no cover
            f.write(("\n" + line + "\n").encode("utf-8"))


def load_var_value_from_shell_script_content(content):
    """
    Extract variable definition such as ``export var="value"`` from shell script
    content text.

    :type content: str
    :param content: text content of a shell script

    :rtype: dict
    :return:
    """
    # find possible text token
    candidate_line_list = list()
    for line in content.split("\n"):
        line_striped = line.strip()
        for item in re.findall(export_pattern, line_striped):
            if item == line:
                candidate_line_list.append(item)

    results = dict()
    for item in candidate_line_list:
        # validate line
        if item.startswith("export ") and ('="' in item) and item.endswith('"'):
            item = item[7:]
            key, value = item.split("=", 1)
            value = value[1:-1]
            results[key] = value

    return results


def load_var_value_from_shell_script(shell_scripts):
    if not os.path.exists(shell_scripts):
        return {}
    with open(shell_scripts, "rb") as f:
        content = f.read().decode("utf-8")
        return load_var_value_from_shell_script_content(content)
